// packages needed in this file
require('dotenv').config()
const express = require('express')
const uri = require('node-uri')
const shortid = require('shortid')

// creating express route handler
const router = express.Router()

// import the Url database model
const Url = require('../models/url')

// @route    POST /api/url/shorten
// @description     Create short URL

// The API base Url endpoint
const baseUrl = process.env.BASEURL

function validUrl(url) {
    let valid
    try {
        uri.checkHttpURL(url)
        valid = true;
    } catch {
        valid = false;
    }
    if (!valid) {
        try {
            uri.checkHttpsURL(url)
            valid = true;
        } catch {
            valid = false;
        }
    }
    return valid;
}

router.post('/shorten', async (req, res) => {
    const {
        longUrl, customPath
    } = req.body // destructure the longUrl from req.body.longUrl

    console.log(longUrl, customPath)
    // check base url if valid using the validUrl method
    
    if (!validUrl(baseUrl)) {
        console.log("Invalid url " + baseUrl)
        return res.status(401).json('Invalid base URL')
    }

    // if valid, we create the url code
    const urlCode = shortid.generate()

    // check long url if valid using the validUrl method
    if (validUrl(longUrl)) {
        try {
            /* The findOne() provides a match to only the subset of the documents 
            in the collection that match the query. In this case, before creating the short URL,
            we check if the long URL was in the DB ,else we create it.
            */
            let url = await Url.findOne({
                longUrl
            })

            // url exist and return the response
            if (url) {
                let json = url.toJSON()
                console.log("Return " + JSON.stringify(json))
                res.json(json)
            } else {
                try {
                    let doc = await Url.findOne({
                        'urlCode': customPath
                    })
                    if (doc) {
                        let json
                        //json["edit"] = true
                        console.log("Found custom path")

                        const shortUrl = baseUrl + '/' + customPath

                        url = new Url({
                            longUrl,
                            shortUrl,
                            urlCode: customPath,
                            date: new Date()
                        })
                        json = url.toJSON()
                        await url.save()

                        let old = await Url.findOneAndUpdate({ urlCode: customPath }, 
                        { urlCode: urlCode, shortUrl:  baseUrl + '/' + urlCode }, 
                        { returnDocument: 'after' })
                        
                        json["old"] = old.toJSON()

                        res.json(json)
                    } else {
                        console.log("Not found custom path")
                        // join the generated short code the the base url
                        const shortUrl = baseUrl + '/' + customPath

                        // invoking the Url model and saving to the DB
                        url = new Url({
                            longUrl,
                            shortUrl,
                            urlCode: customPath,
                            date: new Date()
                        })
                        await url.save()
                        console.log("Return " + url)
                        res.json(url)
                    }
                } catch (err) {
                    console.log(err)
                    res.status(500).json('Server Error')
                }
            }
        }
        // exception handler
        catch (err) {
            console.log(err)
            res.status(500).json('Server Error')
        }
    } else {
        res.status(401).json('Invalid longUrl')
    }
})

module.exports = router
